# pp-finder

Prototype pollution finder tool for javascript. pp-finder lets you find prototype pollution candidates in your code. The main purpose of this tool is to help hackers in there prototype pollution researchs by highlighting potential candidate in the source code.

For instance, the following candidate will be highlighted by pp-finder:

```javascript
if (options.exec) child_process.exec(options.exec);
```

## Installation

```shell
# Install dependencies
$ yarn
$ node dist/index.js --help
```

## Getting started

For instance, let's find candidates in the popular express library:

```
$ mkdir -p target/express
$ cd target/express
$ yarn add express
$ # now write a very basic express application
```

To hunt for prototype pollution gadgets, pp-finder will parse the javascript source code of every file it finds in the provided directory.

It will copy the input directory into a new one with an updated version of the sources.

```shell
$ cd pp-finder/
$ node dist/index.js ../target/express /tmp/express-pp
```

From now, if you go in the `/tmp/express-pp` directory and run the application, it will output all the available candidates:

```
[PP][IsIn "colors"] node_modules/debug/src/node.js:76:22
[PP][ForIn] node_modules/safer-buffer/safer.js:12:13
[PP][IsIn "colors"] node_modules/debug/src/node.js:76:22
[PP][IsIn "colors"] node_modules/debug/src/node.js:76:22
[PP][IsIn "colors"] node_modules/debug/src/node.js:76:22
[PP][Prop noDeprecation] node_modules/depd/index.js:154:15
[...]
Example app listening on port 3000
```

Now, if you issue a request to that server, even more candidate show up:

```
[PP][Prop params] node_modules/express/lib/router/index.js:156:26
[PP][Prop baseUrl] node_modules/express/lib/router/index.js:157:23
[PP][Elem "baseUrl"] node_modules/express/lib/router/index.js:637:19
[PP][Elem "next"] node_modules/express/lib/router/index.js:637:19
[PP][Elem "params"] node_modules/express/lib/router/index.js:637:19
[...]
```

## Configuration

Command line options:

```
❯ pp-finder --help
PP finder
> Add prototype pollution canaries to an existing javascript project.

FLAGS:
  --delete, -d - Delete the output directory before running.
  --help, -h   - show help

OPTIONS:
  --name, -n <str>      - Wrapper name, (default "ø") [optional]
  --exclude, -e <value> - Comma separated list of node-module to exclude [optional]
  --include, -i <value> - Comma separated list of node-module to include [optional]

ARGUMENTS:
  <Out dir>    - Output directory
  <Source dir> - Directory of the targeted application
```

Also, you can configure options at runtime using the environment variables or a config file (`pp-config.json`).

| Options       | Description                                                            | Environement variable | config file |
| :------------ | :--------------------------------------------------------------------- | :-------------------- | :---------- |
| No color      | When set, the output will not be colorized                             | PPF_NO_COLOR          | noColor     |
| Only log once | When set, the same entry will only be displayed once                   | PPF_LOG_ONCE          | onlyNewOnes |
| Output file   | When set, output messages will be written in the specified output file | PPF_OUTPUT_FILE       | outputFile  |

## Tests

Tests indicate what the library handles in terms of AST visitors:

```
❯ yarn run test
yarn run v1.22.19
$ mocha test -r ts-node/register 'test/**/*.test.ts'


  Hooks
    PropertyAccessExpression
      ✔ Pollutable     | ({}).y
      ✔ Not Pollutable | ({y: 42}).y
      ✔ Not Pollutable | (Object.create(null)).y
      ✔ Pollutable     | ({y: {}}).y.z
    ElementAccessExpression
      ✔ Pollutable     | ({})['y']
      ✔ Not Pollutable | ({y: 42})['y']
      ✔ Not Pollutable | (Object.create(null))['y']
      ✔ Pollutable     | ({y: {}})['y']['z']
    ForInStatement
      ✔ Pollutable     | for(let y in ({})){}
      ✔ Not Pollutable | for(let y in (Object.create(null))){}
    ArrowFunctionDeclaration
      ✔ Pollutable     | (({y}) => ())({})
      ✔ Pollutable     | (({y}, a, {z}) => ())({}, 0, {})
      ✔ Pollutable     | (({y: z}) => ())({})
    FunctionDeclaration
      ✔ Pollutable     | function f({y}){return};f({})
      ✔ Pollutable     | function f({y}, a, {z}){return};f({}, 42, {})
      ✔ Pollutable     | function f({y: z}){return};f({})
      ✔ Pollutable     | function f({['y']: z}){return};f({})
    FunctionExpression
      ✔ Pollutable     | (function ({y}){return})({})
      ✔ Pollutable     | (function ({y}, a, {z}){return})({}, 0, {})
      ✔ Pollutable     | (function ({y: z}){return})({})
      ✔ Pollutable     | (function ({['y']: z}){return})({})
    InExpression
      ✔ Pollutable     | ("y" in {})
      ✔ Not Pollutable | ("y" in {y: 42})
      ✔ Not Pollutable | ("y" in Object.create(null))
    ObjectLiteral
      ✔ Pollutable     | ({y} = {});
      ✔ Pollutable     | ({y: {z}} = {y: {}});
      ✔ Not Pollutable | ({y} = {y: 42});
      ✔ Not Pollutable | ({y} = Object.create(null));
      ✔ Pollutable     | ({['y']: y} ={});
    VariableDeclaration
      ✔ Pollutable     | const {y} = {};
      ✔ Pollutable     | const {y} = {}, {z} = {};
      ✔ Pollutable     | const {y: {z}} = {y: {}};
      ✔ Not Pollutable | const {y} = {y: 42};
      ✔ Pollutable     | const {['y']: y} ={};
      ✔ Not Pollutable | const {y} = Object.create(null);
      ✔ Pollutable     | let z; const {y} = {z} = {};


  36 passing (39ms)

Done in 1.78s.


```

## Testing node internals

We can use pp-finder to test prototype pollution within node internals.

To do that, we are going to compile node with extra options .

- [building-nodejs-with-external-core-modules](https://github.com/nodejs/node/blob/main/BUILDING.md#building-nodejs-with-external-core-modules)
- [speeding-up-frequent-rebuilds-when-developing](https://github.com/nodejs/node/blob/main/BUILDING.md#speeding-up-frequent-rebuilds-when-developing)

First, clone node repository:

```shell
git clone --depth=1 https://github.com/nodejs/node.git #  use --depth=1 to avoid pulling all the history
```

Then copy the pp-finder module to the node repo:

```shell
cp ~/worspace/dev/pp-finder/src/module/index.minimal.js ./lib/pp-finder.js
```

And then, configure and compile nodejs:

```
# Don't forget this if you want to speed up the future builds
sudo apt install ccache
export CC="ccache gcc"    # add to your .profile
export CXX="ccache g++"   # add to your .profile

./configure --node-builtin-modules-path "$(pwd)"
make
```

Once compiled, copy the whole node folder to a clean location which will not be written to.

```shell
cp -rf node node-precompiled
```

Then run pp-finder on the lib folder ( containing the js files )

```shell
ts-node pp-finder/src/index.ts -d --mode require --module minimal /tmp/lib ./targets/node-precompiled/lib
```

You can now pick one module to test against like this:

```shell
cp -rf /tmp/lib/internal/fs ./lib/internal
```

And run node:

```shell
./node
```
