"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hookArgs = void 0;
const cmd_ts_1 = require("cmd-ts");
const fs_1 = require("cmd-ts/batteries/fs");
const fs_extra_1 = __importDefault(require("fs-extra"));
const compiler_1 = require("../compiler");
const backupManager_1 = __importDefault(require("../libs/backupManager"));
const utils_1 = require("../libs/utils");
exports.hookArgs = {
    path: (0, cmd_ts_1.positional)({
        type: fs_1.ExistingPath,
        displayName: "Path",
        description: "Path of the file or directory to instrument",
    }),
    minimal: (0, cmd_ts_1.flag)({
        long: "minimal",
        short: "m",
        defaultValue: () => false,
        description: "Use the minimal module",
    }),
    inline: (0, cmd_ts_1.flag)({
        long: "inline",
        defaultValue: () => false,
        description: "Inject pp-finder inline in the beginning of each files",
    }),
    wrapperName: (0, cmd_ts_1.option)({
        type: cmd_ts_1.string,
        long: "name",
        short: "n",
        description: 'Wrapper name, (default "ø")',
        defaultValue: () => "ø",
    }),
};
exports.default = (0, cmd_ts_1.command)({
    name: "hook",
    aliases: ["h"],
    description: "Instrument a javascript project",
    args: exports.hookArgs,
    handler(args) {
        const files = (0, utils_1.getFileNames)(args.path);
        const compiler = (0, compiler_1.createCompiler)(args);
        (0, utils_1.iterWithProgress)(files, (filename) => {
            const { source, isHooked, hash } = (0, utils_1.readFile)(filename);
            // skip already hooked files and cli scripts
            if (isHooked || source.startsWith("#!")) {
                return;
            }
            backupManager_1.default.backup(filename, hash);
            const relativeFilePath = filename.slice(args.path.length + 1);
            try {
                const compiled = compiler.compile(relativeFilePath, source);
                fs_extra_1.default.writeFileSync(filename, compiled + `\n// PPFINDER: ${hash}`);
            }
            catch (e) {
                console.error(`Could not compile file ${relativeFilePath}`);
                console.error(e);
                process.exit(1);
            }
        });
        //Copy pp-finder module
        /*   if (!args.inline) {
          const nodeFolder = path.join(args.path, "node_modules");
          if (!fs.existsSync(nodeFolder)) {
            fs.mkdirSync(nodeFolder);
          }
    
          const modulePath = path.join(path.dirname(__filename), "module");
          const targetModulePath = path.join(nodeFolder, "pp-finder");
    
          fs.copySync(modulePath, targetModulePath);
    
          fs.renameSync(
            path.join(targetModulePath, `index.${args.module}.js`),
            path.join(targetModulePath, `index.js`)
          );
        } */
    },
});
