"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = __importDefault(require("path"));
const zod_1 = require("zod");
const fs_extra_1 = __importDefault(require("fs-extra"));
const os_1 = __importDefault(require("os"));
const backupValidator = zod_1.z.record(zod_1.z.string());
const configFolder = path_1.default.join(os_1.default.homedir(), ".local/share/pp-finder");
const paths = {
    backups: path_1.default.join(configFolder, "backups"),
};
// make sure the backup folder exists
fs_extra_1.default.ensureDir(configFolder);
fs_extra_1.default.ensureDir(paths.backups);
exports.default = {
    purge(filename) {
        fs_extra_1.default.mkdirSync(paths.backups);
        return 0;
    },
    backup(filename, hash) {
        const backupPath = path_1.default.join(paths.backups, hash);
        if (!fs_extra_1.default.existsSync(backupPath)) {
            fs_extra_1.default.copySync(filename, backupPath);
        }
        return hash;
    },
    restore(filename, hash) {
        const backupPath = path_1.default.join(paths.backups, hash);
        if (!fs_extra_1.default.existsSync(backupPath)) {
            return false;
        }
        try {
            fs_extra_1.default.copySync(backupPath, filename, {
                overwrite: true,
            });
        }
        catch (e) {
            console.log(e);
            return false;
        }
        return true;
    },
};
