"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.iterWithProgress = exports.readFile = exports.getFileNames = void 0;
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const crypto_1 = __importDefault(require("crypto"));
function* iterFiles(dirName) {
    function* iterator(dirName) {
        const dir = fs_extra_1.default.opendirSync(dirName);
        let f;
        while ((f = dir.readSync())) {
            const fullPath = path_1.default.join(dirName, f.name);
            if (f.isFile() && path_1.default.extname(f.name) === ".js") {
                yield fullPath;
            }
            if (f.isDirectory()) {
                const parent = path_1.default.basename(dirName);
                if (parent === "node_modules") {
                }
                yield* iterator(fullPath);
            }
        }
        dir.closeSync();
    }
    yield* iterator(dirName);
}
function getFileNames(pathname) {
    const stat = fs_extra_1.default.statSync(pathname);
    const files = [];
    if (stat.isDirectory()) {
        console.log("Computing file list...");
        for (const file of iterFiles(pathname)) {
            files.push(file);
        }
    }
    else {
        files.push(pathname);
    }
    return files;
}
exports.getFileNames = getFileNames;
function isHooked(source) {
    const match = source.match(/^\/\/ PPFINDER: (?<hash>[0-9a-f]{32})$/im);
    if (match) {
        return match.groups.hash;
    }
    return false;
}
function readFile(filename) {
    const source = fs_extra_1.default.readFileSync(filename, {
        encoding: "utf-8",
    });
    const md5 = crypto_1.default.createHash("md5");
    const hash = md5.update(source).digest("hex");
    return { source, hash, isHooked: isHooked(source) };
}
exports.readFile = readFile;
function iterWithProgress(arr, callback) {
    const total = arr.length;
    for (let i = 0; i < total; i++) {
        const item = arr[i];
        callback(item);
        const pct = Math.round(((i + 1) * 100) / total);
        //process.stdout.write(`\r${i + 1}/${total} files ${pct}%`);
    }
    process.stdout.write("\n");
}
exports.iterWithProgress = iterWithProgress;
