"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.objectLiteralTransformer = void 0;
const typescript_1 = __importDefault(require("typescript"));
// ({y} = x);
const objectLiteralTransformer = (node, utils) => {
    // Check
    if (!typescript_1.default.isBinaryExpression(node) ||
        node.operatorToken.kind !== typescript_1.default.SyntaxKind.EqualsToken ||
        !typescript_1.default.isObjectLiteralExpression(node.left)) {
        return null;
    }
    // Transform
    function* iterPaths(node, path = []) {
        for (const prop of node.properties) {
            if (typescript_1.default.isShorthandPropertyAssignment(prop)) {
                yield typescript_1.default.factory.createArrayLiteralExpression([
                    ...path,
                    typescript_1.default.factory.createStringLiteral(prop.name.text),
                ]);
            }
            if (typescript_1.default.isPropertyAssignment(prop) && typescript_1.default.isIdentifier(prop.name)) {
                if (typescript_1.default.isIdentifier(prop.initializer)) {
                    yield typescript_1.default.factory.createArrayLiteralExpression([
                        ...path,
                        typescript_1.default.factory.createStringLiteral(prop.name.text),
                    ]);
                }
                if (typescript_1.default.isObjectLiteralExpression(prop.initializer)) {
                    yield* iterPaths(prop.initializer, [
                        ...path,
                        typescript_1.default.factory.createStringLiteral(prop.name.text),
                    ]);
                }
            }
            if (typescript_1.default.isPropertyAssignment(prop) &&
                typescript_1.default.isComputedPropertyName(prop.name)) {
                yield typescript_1.default.factory.createArrayLiteralExpression([
                    ...path,
                    prop.name.expression,
                ]);
            }
        }
    }
    const paths = Array.from(iterPaths(node.left));
    const newNode = utils.createWrapperCall("bind", node.right, [
        utils.visit(node.right),
        typescript_1.default.factory.createArrayLiteralExpression(paths),
    ]);
    return typescript_1.default.factory.updateBinaryExpression(node, utils.visit(node.left), node.operatorToken, newNode);
};
exports.objectLiteralTransformer = objectLiteralTransformer;
exports.default = exports.objectLiteralTransformer;
