"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceParams = exports.iterBindingPatternPath = exports.isInAssignation = void 0;
const typescript_1 = __importDefault(require("typescript"));
function isAssignation(operator) {
    if (operator === typescript_1.default.SyntaxKind.EqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.PlusEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.MinusEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.AsteriskEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.SlashEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.PercentEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.AmpersandEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.BarEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.CaretEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.LessThanLessThanEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.GreaterThanGreaterThanEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.GreaterThanGreaterThanGreaterThanEqualsToken)
        return true;
    if (operator === typescript_1.default.SyntaxKind.AsteriskAsteriskEqualsToken)
        return true;
    return false;
}
function isInAssignation(node) {
    return (node.parent &&
        typescript_1.default.isBinaryExpression(node.parent) &&
        isAssignation(node.parent.operatorToken.kind) &&
        node.parent.left == node);
}
exports.isInAssignation = isInAssignation;
function* iterBindingPatternPath(node, path = []) {
    for (const elem of node.elements) {
        const x = typescript_1.default.isShorthandPropertyAssignment(elem);
        if (!elem.propertyName) {
            if (typescript_1.default.isIdentifier(elem.name)) {
                yield typescript_1.default.factory.createArrayLiteralExpression([
                    ...path,
                    typescript_1.default.factory.createStringLiteral(elem.name.text),
                ]);
            }
            continue;
        }
        if (typescript_1.default.isIdentifier(elem.propertyName)) {
            if (typescript_1.default.isIdentifier(elem.name)) {
                yield typescript_1.default.factory.createArrayLiteralExpression([
                    ...path,
                    typescript_1.default.factory.createStringLiteral(elem.propertyName.text),
                ]);
            }
            if (typescript_1.default.isObjectBindingPattern(elem.name)) {
                yield* iterBindingPatternPath(elem.name, [
                    ...path,
                    typescript_1.default.factory.createStringLiteral(elem.propertyName.getText()),
                ]);
            }
        }
        if (typescript_1.default.isComputedPropertyName(elem.propertyName)) {
            yield typescript_1.default.factory.createArrayLiteralExpression([
                ...path,
                elem.propertyName.expression,
            ]);
        }
    }
}
exports.iterBindingPatternPath = iterBindingPatternPath;
function replaceParams(node, utils) {
    const newParameters = [];
    const newDeclarations = [];
    for (const param of node.parameters) {
        if (typescript_1.default.isObjectBindingPattern(param.name)) {
            const paths = Array.from(iterBindingPatternPath(param.name));
            const name = `${utils.wrapperName}_${newDeclarations.length}`;
            const identifier = typescript_1.default.factory.createIdentifier(name);
            const newNode = utils.createWrapperCall("bind", param, [
                identifier,
                typescript_1.default.factory.createArrayLiteralExpression(paths),
            ]);
            newDeclarations.push(typescript_1.default.factory.createVariableDeclaration(param.name, undefined, undefined, newNode));
            newParameters.push(typescript_1.default.factory.updateParameterDeclaration(param, param.modifiers, param.dotDotDotToken, identifier, param.questionToken, param.type, param.initializer));
        }
        else {
            newParameters.push(param);
        }
    }
    let newStatements = [];
    if (newDeclarations.length !== 0) {
        newStatements.push(typescript_1.default.factory.createVariableStatement(undefined, typescript_1.default.factory.createVariableDeclarationList(newDeclarations, typescript_1.default.NodeFlags.Let)));
    }
    return { newParameters, newStatements };
}
exports.replaceParams = replaceParams;
